require 'test_setup'

class AuthTest
  include Minispec

  class BasicAuth < Air

    use Rack::Auth::Basic do |u, p|
      [u, p] == ['user', 'pass']
    end

    def index
      __method__
    end

    def post_index
      __method__
    end
  end

  class DigestAuth < Air

    use Rack::Auth::Digest::MD5, 'AccessRestricted', rand.to_s do |u|
      {'digest-user' => 'digest-pass'}[u]
    end

    def index
      __method__
    end

    def post_index
      __method__
    end
  end

  helper :authenticated? do |_,body|
    is(last_response).ok?
    assert(last_response.body) == (body || 'index')
  end

  helper :restricted? do
    assert(last_response.status) == 401
  end

  context :Basic do
    before_all { app BasicAuth }

    it 'returns 401 when unauthorized' do
      get
      is.restricted?
    end

    should 'authenticate and return 200' do
      auth 'user', 'pass'
      get
      is.authenticated?
    end

    testing 'reset_auth!' do
      reset_auth!
      get
      is.restricted?
    end

    should 'relogin after reset_auth! used' do
      auth 'user', 'pass'
      get
      is.authenticated?
    end

    testing 'reset_basic_auth!' do
      reset_basic_auth!
      get
      is.restricted?
    end

    should 'relogin after reset_basic_auth! used' do
      auth 'user', 'pass'
      get
      is.authenticated?
    end

    should 'fail with wrong credentials' do
      reset_basic_auth!
      auth 'bad', 'guy'
      get
      is.restricted?
    end

    should 'auth via POST' do
      reset_basic_auth!
      auth 'user', 'pass'
      post
      is.authenticated? 'post_index'
    end
  end

  context :Digest do
    before_all { app DigestAuth }

    it 'is restricted by default' do
      get
      is.restricted?
    end

    should 'authenticate and return 200' do
      digest_auth 'digest-user', 'digest-pass'
      get
      is.authenticated?
    end

    should 'reset auth using reset_auth!' do
      reset_auth!
      get
      is.restricted?
    end

    should 'relogin after reset_auth! used' do
      digest_auth 'digest-user', 'digest-pass'
      get
      is.authenticated?
    end

    should 'reset auth using reset_digest_auth!' do
      reset_digest_auth!
       get
      is.restricted?
    end

    should 'relogin after reset_digest_auth! used' do
      digest_auth 'digest-user', 'digest-pass'
      get
      is.authenticated?
    end

    should 'fail with wrong credentials' do
      reset_digest_auth!
      digest_auth 'bad', 'guy'
      get
      is.restricted?
    end

    should 'auth via POST' do
      reset_digest_auth!
      digest_auth 'digest-user', 'digest-pass'
      post
      is.authenticated? 'post_index'
    end
  end
end
