require 'test_setup'

class HeadersTest
  include Minispec

  class App < Air

    def index headers
      env.values_at(*headers.split(',')).inspect
    end

    def post_index
      content_type
    end

    def dry
    end

    def post_dry
    end
  end

  before_all { app(App) }

  testing 'User-Agent' do

    ua = 'Chrome'
    header['User-Agent'] = ua
    get 'HTTP_USER_AGENT'
    is(last_response.body) == [ua].inspect

    ua = 'Safari'
    header['User-Agent'] = ua
    get 'HTTP_USER_AGENT'
    is(last_response.body) == [ua].inspect
  end

  testing 'Content-Type' do
    ct = 'text/plain'
    header['Content-Type'] = ct
    get 'CONTENT_TYPE'
    is(last_response.body) == [ct].inspect
  end

  should 'not override explicit CONTENT_TYPE on POST requests' do
    ct = 'text/plain'
    header['Content-Type'] = ct
    post
    is(last_response.body) == ct
  end

  should 'have 127.0.0.1 as default REMOTE_ADDR' do
    get
    expect(last_request.env['REMOTE_ADDR']) == '127.0.0.1'
  end

  should 'use default SERVER_NAME' do
    get
    expect(last_request.env['SERVER_NAME']) == Rack::Radar::RACK_RADAR__DEFAULT_HOST
  end

  context 'Rack-related headers' do

    it 'should override default rack.input' do
      header['rack.input'] = StringIO.new('someString')
      get :dry
      is(last_request.env['rack.input']) == header['rack.input']
    end

    should 'not add multipart content type on post requests' do
      post :dry
      refute(last_request.env['CONTENT_TYPE']) =~ /x\-www\-form\-urlencoded/
    end

    should 'keep explicitly set rack.errors' do
      errors = StringIO.new
      header['rack.errors'] = errors
      get
      expect { last_request.env['rack.errors'].__id__ } == errors.__id__
    end
  end
end
