require 'test_setup'

class MapTest
  include Minispec

  class App < Air

    def news a1
      a1
    end
  end

  before_all { app(App) }

  testing 'without map' do
    get '/news/1'
    is(last_response).ok?
    is(last_response.body) == '1'
  end

  it 'should work only with arguments' do
    map '/news'
    get 1
    is(last_response).ok?
    is(last_response.body) == '1'
  end

  it 'should work without arguments' do
    map '/news/foo'
    get
    expect(last_response.status) == 200
    is(last_response.body) == 'foo'
  end

  it 'should require full url when map set to nil' do
    map nil
    get
    is(last_response).not_found?

    get 'foo'
    is(last_response).not_found?

    get '/news/foo'
    is(last_response).ok?
  end

  it 'should ignore base URL set by map' do
    map '/blah'

    get 'news/foo'
    is(last_response).not_found?

    get '/news/foo'
    is(last_response).ok?

    get 'http://blah.org/news/foo'
    is(last_response).not_found?

    header['HTTP_HOST'] = 'blah.org'
    get 'http://blah.org/news/foo'
    is(last_response).ok?
  end

end
