require 'test_setup'

class ParamsTest
  include Minispec

  class App < Air

    def index vars
      self.GET.values_at(*vars.split(',')).inspect
    end

    def post_index vars
      self.POST.values_at(*vars.split(',')).inspect
    end
  end

  before_all { app App }
  before do
    @vars, @vals = 2.times.map { 10.times.map { rand.to_s } }
    @params = Hash[@vars.zip(@vals)]
  end

  it 'handle GET params' do
    get @vars.join(','), @params
    is(last_response.body) == @params.values_at(*@vars).inspect
  end

  it 'handle POST params' do
    post @vars.join(','), @params
    expect(last_response.body) == @params.values_at(*@vars).inspect
  end

  should 'mix params [Hash] with query_string [String]' do
    qs_vars, qs_vals = 2.times.map { 10.times.map { rand.to_s } }
    qs_params = Hash[qs_vars.zip qs_vals]
    query_string = ::Rack::Utils.build_query(qs_params)
    get((@vars + qs_vars).join(',') << '?' << query_string, @params)
    is(last_response.body) == @params.merge(qs_params).values_at(*(@vars + qs_vars)).inspect
  end

  should 'have content type set to application/x-www-form-urlencoded' do
    post @vars.join(','), @params
    expect(last_request.env['CONTENT_TYPE']) == 'application/x-www-form-urlencoded'
  end
end
