require 'test_setup'

class RequestMethodsTest
  include Minispec

  class App < Air

    Rack::Radar::RACK_RADAR__REQUEST_METHODS.each do |rm|
      define_method('%s_index' % rm) { body }
    end

    private
    def body
      [request_method, port, ssl?, xhr?].inspect
    end
  end

  before_all { app App }

  testing :generic do
    Rack::Radar::RACK_RADAR__REQUEST_METHODS.each do |rm|
      self.send(rm.downcase)
      is(last_response).ok?
      is(last_response.body) == [rm, 80, false, false].inspect
    end
  end

  testing :SSL do
    Rack::Radar::RACK_RADAR__REQUEST_METHODS.each do |rm|
      self.send('s_' + rm.downcase)
      is(last_response).ok?
      is(last_response.body) == [rm, 443, true, false].inspect
    end
  end

  testing :XHR do
    Rack::Radar::RACK_RADAR__REQUEST_METHODS.each do |rm|
      self.send(rm.downcase + '_x')
      is(last_response).ok?
      is(last_response.body) == [rm, 80, false, true].inspect
    end
  end

  testing :SecureXHR do
    Rack::Radar::RACK_RADAR__REQUEST_METHODS.each do |rm|
      self.send('s_' + rm.downcase + '_x')
      is(last_response).ok?
      is(last_response.body) == [rm, 443, true, true].inspect
    end
  end

end
